<?php

namespace Slovocast\Domain\Repository;

use Slovocast\Infrastructure\DatabaseConnectionInterface;
use Slovocast\Exception\EntityNotFoundException;
use Slovocast\Domain\Entity\User;
use PDO;

class UserRepository implements UserRepositoryInterface
{
    public function __construct(
        private DatabaseConnectionInterface $database
    ) {}

    /**
     * @param array $results The Query Results asking for all properties from
     *                       the database tables.
     * @return User
     */
    protected function userFromQueryResults(array $results): User
    {
        return User::fromArray([
            'id' => $results['id'],
            'email' => $results['email'],
            'password' => $results['password'],
            'name' => $results['name'],
            'createdAt' => $results['created_at'],
            'updatedAt' => $results['updated_at']
        ]);
    }

    /**
     * @param string $query The Query for getting a User
     * @param arary $params The parameters in the query
     * @param PDO $connection The PDO connection
     * @return array The column data from the Database
     * @throws EntityNotFoundException The User does no exist
     */
    protected function queryForUser(
        string $query,
        array $params,
        PDO $connection
    ): array {
        $statement = $connection->prepare($query);
        $statement->execute($params);
        $results = $statement->fetchAll();

        if (!$results) {
            throw new EntityNotFoundException("Unable to find User");
        }

        return array_shift($results);
    }

    public function get(int $id): User
    {
        $query = "SELECT * FROM users WHERE id = :id LIMIT 1";
        $connection = $this->database->getConnection();
        $userData = $this->queryForUser(
            $query,
            [ 'id' => $id ],
            $connection
        );
        return $this->userFromQueryResults($userData);
    }

    public function getFromEmail(string $email): User
    {
        $query = "SELECT * FROM users WHERE email = :email LIMIT 1";
        $connection = $this->database->getConnection();
        $userData = $this->queryForUser(
            $query, 
            [ 'email' => $email ], 
            $connection
        );
        return $this->userFromQueryResults($userData);
    }
}
